@@ -105,6 +105,7 @@ unique_message_data_copy (UniqueMessageData *message_data)
     }
 
   retval->screen = message_data->screen;
+  retval->startup_id = g_strdup (message_data->startup_id);
 
   return retval;
 }
@@ -120,6 +121,7 @@ unique_message_data_free (UniqueMessageData *message_data)
 {
   if (G_LIKELY (message_data))
     {
+      g_free (message_data->startup_id);
       g_free (message_data->data);
 
       g_slice_free (UniqueMessageData, message_data);
@@ -485,7 +487,7 @@ unique_message_data_get_filename (UniqueMessageData *message_data)
 {
   g_return_val_if_fail (message_data != NULL, NULL);
 
-  return g_strndup ((const gchar *) message_data->data, message_data->length);
+  return g_memdup (message_data->data, message_data->length);
 }
 
 /**
